/*
 * Filename: dipio__infra.h
 *
 * (C) Copyright 2000                    Pattern Recognition Group
 *     All rights reserved               Faculty of Applied Physics
 *                                       Delft University of Technology
 *                                       Lorentzweg 1
 *                                       2628 CJ Delft
 *                                       The Netherlands
 *
 * Contact: Dr. ir. Lucas J. van Vliet
 * email : lucas@ph.tn.tudelft.nl
 *
 */

#ifndef DIPIO__INFRA_H
#define DIPIO__INFRA_H

#include "dipio.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef struct
{
   dip_int size;
   dipio_ImageReadRegistry *array;
} dipio__ImageReadRegistryArray, *dipio_ImageReadRegistryArray;

typedef struct
{
   dip_int size;
   dipio_ImageWriteRegistry *array;
} dipio__ImageWriteRegistryArray, *dipio_ImageWriteRegistryArray;

typedef struct
{
   dip_int size;
   dipio_MeasurementReadRegistry *array;
} dipio__MeasurementReadRegistryArray, *dipio_MeasurementReadRegistryArray;

typedef struct
{
   dip_int size;
   dipio_MeasurementWriteRegistry *array;
} dipio__MeasurementWriteRegistryArray, *dipio_MeasurementWriteRegistryArray;


dip_Error dipio_ImageReadRegistryArrayNew
	( dipio_ImageReadRegistryArray *, dip_int, dip_Resources );
dip_Error dipio_ImageWriteRegistryArrayNew
	( dipio_ImageWriteRegistryArray *, dip_int, dip_Resources );

dip_Error dipio_MeasurementReadRegistryArrayNew
	( dipio_MeasurementReadRegistryArray *, dip_int, dip_Resources );
dip_Error dipio_MeasurementWriteRegistryArrayNew
	( dipio_MeasurementWriteRegistryArray *, dip_int, dip_Resources );


DIPIO_ERROR dipio_ForgeImageAndGetDataPointer (dip_Image,
						dip_IntegerArray, dip_DataType, void**);

DIPIO_EXPORT void dipio__ReadBuffer8      (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray);
DIPIO_EXPORT void dipio__ReadBuffer       (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray, dip_uint);
DIPIO_EXPORT void dipio__ReadBuffer3D     (void*, const void*, dip_uint, dip_uint,
                                           dip_uint, dip_IntegerArray, dip_uint);
DIPIO_EXPORT void dipio__ExpandNibbles    (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray);
DIPIO_EXPORT void dipio__ExpandBits       (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray);
DIPIO_EXPORT void dipio__ExpandColourMap4 (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray,
                                           dip_uint8*, dip_uint8*, dip_uint8*);
DIPIO_EXPORT void dipio__ExpandColourMap8 (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray,
                                           dip_uint8*, dip_uint8*, dip_uint8*);
DIPIO_EXPORT void dipio__FillBuffer8      (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray);
DIPIO_EXPORT void dipio__FillBuffer       (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray, dip_uint);
DIPIO_EXPORT void dipio__CompactBits8     (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray, dip_int);
DIPIO_EXPORT void dipio__CompactBits16    (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray, dip_int);
DIPIO_EXPORT void dipio__CompactBits32    (void*, const void*, dip_uint, dip_uint,
                                           dip_IntegerArray, dip_int);

#define dipio_IntCeilDiv(a,b) ((((a)-((a)/(b))*(b))>0)?(((a)/(b))+1):((a)/(b)))

#ifdef __cplusplus
}
#endif


#endif /* DIPIO__INFRA_H */
