%INV   Inverse of a tensor image
%   INV(A) returns the inverse of A
%   in the sense that INV(A)*A gives a tensor
%   image that has images with only 1 on the diagonal
%   and images with 0 on the off-diagonal
%   NOTE: only implemented for 2x2 and 3x3 tensor images


% (C) Copyright 1999-2000               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, November 2000.
% Nov, 2002, added 4x4 inverse (BR)

function out = inv(a)

if ~istensor(a)
   error('Image is not a tensor image.');
end
s = builtin('size',a);
if length(s) ~= 2 | s(1)~=s(2)
   error('Tensor image must be square to invert.');
end


wa = warning;
warning('off');
switch s(1)
 case 2
   out = dip_image('array',[2,2]);
   out(1,1) = a(2,2);
   out(1,2) = -a(1,2);
   out(2,1) = -a(2,1);
   out(2,2) = a(1,1);
   out = out./det(a);
 case 3
   out = dip_image('array',[3,3]);
   %Kramer Regel
   out(1,1) =   a(2,2).*a(3,3)-a(2,3).*a(3,2);
   out(1,2) = -(a(1,2).*a(3,3)-a(1,3).*a(3,2));
   out(1,3) =   a(1,2).*a(2,3)-a(1,3).*a(2,2);
   out(2,1) = -(a(2,1).*a(3,3)-a(2,3).*a(3,1));
   out(2,2) =   a(1,1).*a(3,3)-a(1,3).*a(3,1);
   out(2,3) = -(a(1,1).*a(2,3)-a(1,3).*a(2,1));
   out(3,1) = -(a(2,2).*a(3,1)-a(2,1).*a(3,2));
   out(3,2) = -(a(1,1).*a(3,2)-a(1,2).*a(3,1));
   out(3,3) =   a(1,1).*a(2,2)-a(1,2).*a(2,1);
   out = out./det(a);
 case 4
   out = dip_image('array',[4,4]);
   out(1,1) =   (a(2,2)*a(3,3)*a(4,4) - a(2,2)*a(3,4)*a(4,3) - ...
                 a(3,2)*a(2,3)*a(4,4) + a(3,2)*a(2,4)*a(4,3) + ...
                 a(4,2)*a(2,3)*a(3,4) - a(4,2)*a(2,4)*a(3,3));
   out(1,2) = - (a(1,2)*a(3,3)*a(4,4) - a(1,2)*a(3,4)*a(4,3) - ...
                 a(3,2)*a(1,3)*a(4,4) + a(3,2)*a(1,4)*a(4,3) + ...
                 a(4,2)*a(1,3)*a(3,4) - a(4,2)*a(1,4)*a(3,3));
   out(1,3) = -(-a(1,2)*a(2,3)*a(4,4) + a(1,2)*a(2,4)*a(4,3) + ...
                 a(2,2)*a(1,3)*a(4,4) - a(2,2)*a(1,4)*a(4,3) - ...
                 a(4,2)*a(1,3)*a(2,4) + a(4,2)*a(1,4)*a(2,3));
   out(1,4) = - (a(1,2)*a(2,3)*a(3,4) - a(1,2)*a(2,4)*a(3,3) - ...
                 a(2,2)*a(1,3)*a(3,4) + a(2,2)*a(1,4)*a(3,3) + ...
                 a(3,2)*a(1,3)*a(2,4) - a(3,2)*a(1,4)*a(2,3));
   out(2,1) = - (a(2,1)*a(3,3)*a(4,4) - a(2,1)*a(3,4)*a(4,3) - ...
                 a(3,1)*a(2,3)*a(4,4) + a(3,1)*a(2,4)*a(4,3) + ...
                 a(4,1)*a(2,3)*a(3,4) - a(4,1)*a(2,4)*a(3,3));
   out(2,2) =   (a(1,1)*a(3,3)*a(4,4) - a(1,1)*a(3,4)*a(4,3) - ...
                 a(3,1)*a(1,3)*a(4,4) + a(3,1)*a(1,4)*a(4,3) + ...
                 a(4,1)*a(1,3)*a(3,4) - a(4,1)*a(1,4)*a(3,3));
   out(2,3) = - (a(1,1)*a(2,3)*a(4,4) - a(1,1)*a(2,4)*a(4,3) - ...
                 a(2,1)*a(1,3)*a(4,4) + a(2,1)*a(1,4)*a(4,3) + ...
                 a(4,1)*a(1,3)*a(2,4) - a(4,1)*a(1,4)*a(2,3));
   out(2,4) = -(-a(1,1)*a(2,3)*a(3,4) + a(1,1)*a(2,4)*a(3,3) + ...
                 a(2,1)*a(1,3)*a(3,4) - a(2,1)*a(1,4)*a(3,3) - ...
                 a(3,1)*a(1,3)*a(2,4) + a(3,1)*a(1,4)*a(2,3));
   out(3,1) = -(-a(2,1)*a(3,2)*a(4,4) + a(2,1)*a(3,4)*a(4,2) + ...
                 a(3,1)*a(2,2)*a(4,4) - a(3,1)*a(2,4)*a(4,2) - ...
                 a(4,1)*a(2,2)*a(3,4) + a(4,1)*a(2,4)*a(3,2));
   out(3,2) = - (a(1,1)*a(3,2)*a(4,4) - a(1,1)*a(3,4)*a(4,2) - ...
                 a(3,1)*a(1,2)*a(4,4) + a(3,1)*a(1,4)*a(4,2) + ...
                 a(4,1)*a(1,2)*a(3,4) - a(4,1)*a(1,4)*a(3,2));
   out(3,3) =   (a(1,1)*a(2,2)*a(4,4) - a(1,1)*a(2,4)*a(4,2) - ...
                 a(2,1)*a(1,2)*a(4,4) + a(2,1)*a(1,4)*a(4,2) + ...
                 a(4,1)*a(1,2)*a(2,4) - a(4,1)*a(1,4)*a(2,2));
   out(3,4) = - (a(1,1)*a(2,2)*a(3,4) - a(1,1)*a(2,4)*a(3,2) - ...
                 a(2,1)*a(1,2)*a(3,4) + a(2,1)*a(1,4)*a(3,2) + ...
                 a(3,1)*a(1,2)*a(2,4) - a(3,1)*a(1,4)*a(2,2));
   out(4,1) = - (a(2,1)*a(3,2)*a(4,3) - a(2,1)*a(3,3)*a(4,2) - ...
                 a(3,1)*a(2,2)*a(4,3) + a(3,1)*a(2,3)*a(4,2) + ...
                 a(4,1)*a(2,2)*a(3,3) - a(4,1)*a(2,3)*a(3,2));
   out(4,2) = -(-a(1,1)*a(3,2)*a(4,3) + a(1,1)*a(3,3)*a(4,2) + ...
                 a(3,1)*a(1,2)*a(4,3) - a(3,1)*a(1,3)*a(4,2) - ...
                 a(4,1)*a(1,2)*a(3,3) + a(4,1)*a(1,3)*a(3,2));
   out(4,3) = - (a(1,1)*a(2,2)*a(4,3) - a(1,1)*a(2,3)*a(4,2) - ...
                 a(2,1)*a(1,2)*a(4,3) + a(2,1)*a(1,3)*a(4,2) + ...
                 a(4,1)*a(1,2)*a(2,3) - a(4,1)*a(1,3)*a(2,2));
   out(4,4) =   (a(1,1)*a(2,2)*a(3,3) - a(1,1)*a(2,3)*a(3,2) - ...
                 a(2,1)*a(1,2)*a(3,3) + a(2,1)*a(1,3)*a(3,2) + ...
                 a(3,1)*a(1,2)*a(2,3) - a(3,1)*a(1,3)*a(2,2));
   out = out./det(a);
 otherwise
   error('Only supported for tensor images of size up to 4x4!');
end
warning(wa);
