%MEAN   Mean of all pixels in an image.
%   VALUE = MEAN(B) returns the mean intensity of all pixels in image B.
%
%   VALUE = MEAN(B,M), with M a binary image, is the same as MEAN(B(M)).
%
%   VALUE = MEAN(B,[],DIM) performs the computation over the dimensions
%   specified in DIM. DIM can be an array with any number of dimensions.
%   M can be [].

% (C) Copyright 1999-2001               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, April 2001.

function value = mean(in,mask,d)
if prod(builtin('size',in)) ~= 1
   error('Parameter "in" is an array of images.')
end
nd = ndims(in);
if nd == 0
   value = double(in);
else
   if nargin < 2
      mask = [];
   end
   try
      if nargin < 3
         process = di_processarray(nd);
         value = double(dip_mean(in,mask,process));
      else
         process = di_processarray(nd,d);
         value = dip_mean(in,mask,process);
      end
   catch
      error(di_firsterr)
   end
end
