%FINDSHIFT  Finds shift of two images
%
% SYNOPSIS:
%  image_out = findshift(in1, in2, method, parameter)
%
% PARAMETERS:
%  methode = 'integer': find integer shift only
%            'ffts'   : 2 step correlation
%            'grs'    : gradient based (first order Taylor)
%  para = scalar paramter for 'ffts', 'grs'
%
% DEFAULTS:
%  methode = 'integer'
%  parameter = 0
%
% LITERATURE:
%  C.L. Luengo Hendriks, Improved Resolution in Infrared Imaging Using
%    Randomly Shifted Images, M.Sc. Thesis, Delft University of Technology, 1998

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, March 2002.

function out = findshift(varargin)

d = struct('menu','Analysis',...
  'display','Find shift',...
  'inparams',struct('name',{'in1', 'in2',  'method','para'},...
       'description',{'Input image 1','Input image 2','Method','Paramter'},...
       'type',       {'image','image','option', 'array'},...
       'dim_check',  {0,0,0,0},...
       'range_check',{[],[],{'integer','ffts','grs'},'R+'},...
       'required',   {1,1,0,0},...
       'default',    {'a', 'b','integer',0}...
      ),...
  'outparams',struct('name',{'out'},...
      'description',{'Shift'},...
      'type',{'array'}...
      )...
 );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in1,in2, method, para] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
out = dip_findshift(in1,in2,method,para);
