%GAUSS_DERIVATIVE   Gaussian derivatives
%
% SYNOPSIS 2D:
%  image_out = gauss_derivative(image_in,sigma,[order_x,order_y])
%
% SYNOPSIS 3D:
%  image_out = gauss_derivative(image_in,sigma,[order_x,order_y,order_z])
%
% DEFAULTS:
%  sigma = 1
%  order = [1,1] for 2D images, [1,1,1] for 3D.

% (C) Copyright 1999-2000               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Lucas van Vliet, May 2000.

function image_out = gauss_derivative(varargin)

d = struct('menu','Differential Filters',...
           'display','General Gaussian derivatives',...
           'inparams',struct('name',       {'image_in',   'sigma',            'order'},...
                             'description',{'Input image','Sigma of gaussian','Derivative Order'},...
                             'type',       {'image',      'array',            'array'},...
                             'dim_check',  {0,            1,                  1},...
                             'range_check',{[],           'R+',               'N'},...
                             'required',   {1,            0,                  0},...
                             'default',    {'a',          1,                  1}...
                            ),...
           'outparams',struct('name',{'image_out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out = d;
      return
   end
end
try
   [image_in,sigma,order] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
process = ones(size(sigma));

image_out = dip_gauss(image_in,process,sigma,order);
