%GDT   Grey-Weighted Distance Transform
%
% SYNOPSIS:
%  [image_out,distance_out] = gdt(image_in,image_seed,chamfer)
%
% PARAMETERS:
%  chamfer: Chamfer distance of 3 or 5.
%
% DEFAULTS:
%  chamfer = 3

% (C) Copyright 1999-2001               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Lucas van Vliet, May 2000.
% Changed parameter order: Cris Luengo, 12 June 2001.

function [image_out1, image_out2] = gdt(varargin)

d = struct('menu','Transforms',...
           'display','Grey-Weighted Distance Transform',...
           'inparams',struct('name',       {'image_in',            'image_weight','chamfer'},...
                             'description',{'Input image (binary)','Weight image','Chamfer Size'},...
                             'type',       {'image',               'image',       'option'},...
                             'dim_check',  {0,                     1,             0},...
                             'range_check',{[],                    [],            {3,5}},...
                             'required',   {1,                     1,             0},...
                             'default',    {'a',                   'b',           3}...
                            ),...
           'outparams',struct('name',       {'image_out1','image_out2'},...
                              'description',{'Output image','Distance Image'},...
                              'type',       {'image','image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out1 = d;
      return
   end
end
try
   [image_in,image_weight,chamfer] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

[image_out1,image_out2] = dip_gdt(image_weight,image_in,chamfer,[],[]);
