%SCALESPACE   Gaussian scale-space
%
% SYNOPSIS:
%  [sp,bp,pp] = scalespace(image_in,nscales,base)
%
% DEFAULTS:
%  nscales = 7
%  base = 2

% (C) Copyright 1999-2000               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Peter Bakker, Oct 2000.

function [sp,bp,pp] = scalespace(varargin)

d = struct('menu','Analysis',...
           'display','Gaussian scale-space',...
           'inparams',struct('name',       {'image_in',   'scales','base'},...
                             'description',{'Input image','Scales','Base'},...
                             'type',       {'image',      'array', 'array'},...
                             'dim_check',  {0,            0,       0},...
                             'range_check',{[],           'N+',    'R+'},...
                             'required',   {1,            0,       0},...
                             'default',    {'a',          7,       2.0}...
                            ),...
           'outparams',struct('name',       {'im_out1','im_out2','im_out3'},...
                              'description',{'Scale','Difference','Variance'},...
                              'type',       {'image','image','image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      sp = d;
      return
   end
end
try
   [image_in,scales,base] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

%
% My code
%

if ndims(image_in) ~= 2
   error('General PB Error.')
end

process = [1,1];
order = [0,0];
scale1 = [1.0,1.0];

sp = newim([size(image_in),scales]);
bp = newim([size(image_in),(scales-1)]);
pp = newim([size(image_in),(scales-1)]);

sp(:,:,0) = image_in;

for ii=1 : scales-1
   scale2 = scale1*(base^(ii-1));
   sp(:,:,ii) = dip_gauss(image_in,process,scale2,order);
   bp(:,:,(ii-1)) = sp(:,:,ii) - sp(:,:,ii-1);
   pp(:,:,(ii-1)) = bp(:,:,ii-1)*bp(:,:,ii-1);
   pp(:,:,(ii-1)) = dip_gauss(squeeze(pp(:,:,(ii-1))),process,scale2,order);
end
